/*
 * DIP-Switch Demo for DIL/NetPC DNP/5280-3V
 * Author  : MHA@SSV <info@dilnetpc.com> <m.hasewinkel@web.de>
 * Date    : 26.08.2003
 * Version : 1.00
 * License : GPL
 */

#include <unistd.h>
#include <stdio.h>

#include "ssvhwa.h"

#define MCFBAR  0x40000000

#define PORTQA   (MCFBAR + 0x00190006)	/* 8 bit */
#define PORTQB   (MCFBAR + 0x00190007)	/* 8 bit */
#define DDRQA    (MCFBAR + 0x00190008)	/* 8 bit */
#define DDRQB    (MCFBAR + 0x00190009)	/* 8 bit */

#define PORTAS   (MCFBAR + 0x0010000C)	/* 8 bit */
#define DDRAS    (MCFBAR + 0x00100020)	/* 8 bit */
#define PORTASP  (MCFBAR + 0x00100034)	/* 8 bit */
#define SETAS    (MCFBAR + 0x00100034)	/* 8 bit */
#define CLRAS    (MCFBAR + 0x00100048)	/* 8 bit */
#define PASPAR   (MCFBAR + 0x00100056)	/* 16 bit */

#define GPTAPORT (MCFBAR + 0x001A001D)	/* 8 bit */
#define GPTADDR  (MCFBAR + 0x001A001E)	/* 8 bit */

void write_portA(unsigned char data)
{
    // write data Port A[0-3]
    ssvhwa_write8(PORTQA, ((data & 0x0c) << 1) | (data & 0x03));
    // write data Port A[4-7]
    ssvhwa_write8(PORTQB, (data >> 4));
}

unsigned char read_portB(void)
{
    unsigned char data_bl, data_bh;

    // read data Port B[0-3]
    data_bl = ssvhwa_read8(GPTAPORT) & 0xf;
    // read data Port B[4-7]
    data_bh = (ssvhwa_read8(PORTASP) & 0xf) << 4;

    return(data_bh | data_bl);
}

int main (void)
{
    unsigned char iDIPsw, iDIPswMirror;

    // check user identity
    if (geteuid() != 0) {
	printf("No root access rights !\n");
	exit(1);
    }

    if (ssvhwa_open() < 0) {		// open ssvhwa driver
	perror("ssvhwa open");
	exit(-1);
    }

    // set DIL/NetPC PIO Port A = output
    ssvhwa_write8(DDRQA, 0x1b);		// Port A[0-3] output
    ssvhwa_write8(DDRQB, 0x0f);		// Port A[4-7] output

    // set DIL/NetPC PIO Port B = input
    ssvhwa_write8(GPTADDR, 0x00);	//Port B[0-3] input
    ssvhwa_write8(DDRAS, 0x00);		//Port B[7-4] input

    // build DIP-Switch status mirror
    iDIPswMirror = read_portB();	// read Port B

    // show current DIP-Switch status (message and LEDs)...
    printf ("\n  DIP-Switch= 0x%02x", iDIPswMirror);
    fflush(stdout);

    // update LEDs
    write_portA(iDIPswMirror);		// write Port A

    // read DIP-Switch and show status until user break...

    while (1) {

	// Read current DIP-Switch Status.
	iDIPsw = read_portB();			// read Port B

	// compare current status with last status
	if (iDIPsw != iDIPswMirror) {

	    // DIP-Switch status change...
	    iDIPswMirror= iDIPsw;
	    printf ("\b\b%02x", iDIPsw);	// update message
	    fflush(stdout);

	    // update LEDs
	    write_portA(iDIPsw);		// write Port A

	}
	usleep(1000);
    }

    ssvhwa_close();

    return(0);
}
